/* -----------------------------------------------------------------------------
 * Copyright (c) 2014 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty. 
 * In no event will the authors be held liable for any damages arising from 
 * the use of this software. Permission is granted to anyone to use this 
 * software for any purpose, including commercial applications, and to alter 
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not 
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be 
 *    appreciated but is not required. 
 * 
 * 2. Altered source versions must be plainly marked as such, and must not be 
 *    misrepresented as being the original software. 
 * 
 * 3. This notice may not be removed or altered from any source distribution.
 *   
 *
 * $Date:        15. April 2014
 * $Revision:    V1.00
 *  
 * Project:      Flash Programming Functions for ST Microelectronics STM32F0xx Flash
 * --------------------------------------------------------------------------- */

/* History:
 *  Version 1.00
 *    Initial release
 */ 

#include "..\FlashOS.H"        // FlashOS Structures

typedef volatile unsigned char    vu8;
typedef          unsigned char     u8;
typedef volatile unsigned short   vu16;
typedef          unsigned short    u16;
typedef volatile unsigned long    vu32;
typedef          unsigned long     u32;

#define M8(adr)  (*((vu8  *) (adr)))
#define M16(adr) (*((vu16 *) (adr)))
#define M32(adr) (*((vu32 *) (adr)))

// Registers_BLUENRG
#define NVM_REG_COMMAND         ((unsigned long)0x40100000 + 0x00)    // 0x00
#define NVM_REG_CONFIG          ((unsigned long)0x40100000 + 0x04)    // 0x04
#define NVM_REG_IRQSTAT         ((unsigned long)0x40100000 + 0x08)    // 0x08
#define NVM_REG_IRQMASK         ((unsigned long)0x40100000 + 0x0C)    // 0x0C
#define NVM_REG_IRQRAW          ((unsigned long)0x40100000 + 0x10)    // 0x10
#define NVM_REG_FLASH_SIZE      ((unsigned long)0x40100000 + 0x14)    // 0x14
#define NVM_REG_ADDRESS         ((unsigned long)0x40100000 + 0x18)    // 0x18
#define NVM_REG_UNLOCKM         ((unsigned long)0x40100000 + 0x1C)    // 0x1C
#define NVM_REG_UNLOCKL         ((unsigned long)0x40100000 + 0x20)    // 0x20
#define NVM_REG_LFSRVAL         ((unsigned long)0x40100000 + 0x24)    // 0x24
#define NVM_REG_TIMETRIM1       ((unsigned long)0x40100000 + 0x28)    // 0x28
#define NVM_REG_TIMETRIM2       ((unsigned long)0x40100000 + 0x2C)    // 0x2C
#define NVM_REG_DATA            ((unsigned long)0x40100000 + 0x40)    // 0x40
// Bit fields_BLUENRG
#define NVM_IRQ_CMDDONE         0x01
#define NVM_IRQ_CMDBUSY         0x02
#define NVM_IRQ_CMDERR          0x04
#define NVM_IRQ_ILLCMD          0x08
#define NVM_IRQ_READOK          0x10
#define NVM_IRQ_FLNREADY        0x20

#define NVM_CMD_ERASE           0x11
#define NVM_CMD_MASSERASE       0x22
#define NVM_CMD_WRITE           0x33
#define NVM_CMD_MASSWRITE       0x44
#define NVM_CMD_MASSREAD        0x55
#define NVM_CMD_IFRERASE        0x66
#define NVM_CMD_IFRWRITE        0x77
#define NVM_CMD_IFRMASSWRITE    0x88
#define NVM_CMD_IFRMASSREAD     0x99
#define NVM_CMD_SLEEP           0xAA
#define NVM_CMD_WAKEUP          0xBB
#define NVM_CMD_BURSTWRITE      0xCC
#define NVM_CMD_IFRBURSTWRITE   0xDD
#define NVM_CMDSTART            0x00000002
#define NVM_CMDDONE             0x00000001


/*
 *  Initialize Flash Programming Functions
 *    Parameter:      adr:  Device Base Address
 *                    clk:  Clock Frequency (Hz)
 *                    fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

#ifdef FLASH_MEM
int Init (unsigned long adr, unsigned long clk, unsigned long fnc) {

/*  Flash Unlock */
  *((unsigned long *)NVM_REG_UNLOCKM) = 0x1002FFF8;
  *((unsigned long *)NVM_REG_UNLOCKL) = 0x1002FFFC;
  
  /* Clear status */
  *((unsigned long *)NVM_REG_IRQSTAT) = 0xFFFFFFFF;
  *((unsigned long *)0x200000C0)=0x0;

  return (0);
}
#endif

/*
 *  De-Initialize Flash Programming Functions
 *    Parameter:      fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

#ifdef FLASH_MEM
int UnInit (unsigned long fnc) {

 // FLASH->CR    |=  FLASH_LOCK;                          // Lock Flash

  return (0);
}
#endif

/*
 *  Erase complete Flash Memory
 *    Return Value:   0 - OK,  1 - Failed
 */

#ifdef FLASH_MEM
int EraseChip (void) {
 
	
	 /* Clear status */
  *((unsigned long*)NVM_REG_IRQSTAT) = 0xFFFFFFFF;
   
  /* Mass erase command */
  *((unsigned long*)NVM_REG_COMMAND) = NVM_CMD_MASSERASE;
 
   unsigned long stat = *((unsigned long*)NVM_REG_IRQSTAT);
	while (!(stat & NVM_CMDSTART));
	*((unsigned long*)NVM_REG_IRQSTAT)|= NVM_CMDSTART;
	stat = *((unsigned long*)NVM_REG_IRQSTAT);
	
	while (!(stat & NVM_CMDDONE));
	*((unsigned long*)NVM_REG_IRQSTAT)|= NVM_CMDSTART;

 
    return (0);                                    
}
#endif


/*
 *  Erase Sector in Flash Memory
 *    Parameter:      adr:  Sector Address
 *    Return Value:   0 - OK,  1 - Failed
 */

#ifdef FLASH_MEM
int EraseSector (unsigned long adr) {
    *((unsigned long*)NVM_REG_ADDRESS) = (adr - 0x10040000)/4;
    *((unsigned long*)NVM_REG_COMMAND) =  NVM_CMD_ERASE;
	 for (int i=0; i<50000; i++); 
  
  unsigned long stat = *((unsigned long*)NVM_REG_IRQSTAT);
  if( stat & (NVM_IRQ_CMDERR | NVM_IRQ_ILLCMD) )  // error
     return (1);
  
	return (0);                                           // Done
}
#endif


/*
 *  Program Page in Flash Memory
 *    Parameter:      adr:  Page Start Address
 *                    sz:   Page Size
 *                    buf:  Page Data
 *    Return Value:   0 - OK,  1 - Failed
 */

#ifdef FLASH_MEM
int ProgramPage (unsigned long Address, unsigned long Size, unsigned char *buffer) {

  int Addr = (Address - 0x10040000)/4;
	  

  for( ; Size; Size-=4)
  { 
    /* Write Address offset */
    *((unsigned long*)NVM_REG_ADDRESS) = Addr;
   
    /* short Delay */
    for (int i=0; i<250; i++);
    
    /* Write Data */
    *((unsigned long*)NVM_REG_DATA) = *((unsigned long *)buffer); ;

    /* short Delay */
    for (int i=0; i<250; i++);
  
    /* Write command */
    *((unsigned long*)NVM_REG_COMMAND) = NVM_CMD_WRITE;
    
    unsigned long stat = *((unsigned long*)NVM_REG_ADDRESS);
    
    /* wait for complete operation */
    while(stat && (NVM_IRQ_CMDDONE|NVM_IRQ_CMDBUSY) != 0x3)
    {
    }
    
    /* short Delay */
    for (int i=0; i<250; i++);
 
    Addr++;
    buffer+=4;
  }
  
  unsigned long stat = *((unsigned long*)NVM_REG_IRQSTAT);
  if( stat & (NVM_IRQ_CMDERR | NVM_IRQ_ILLCMD) )
	return (1);  										  //failed
  else
	return (0);                                           // Done
}
#endif

